/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 2012, 2018  All Rights Reserved
 */
package java.lang.invoke;

final class MethodHandleInfoImpl {

	/* The following fields contain all the information that
	 * this class's methods will return
	 */
	private final Class declaringClass;
	private final String name;
	private final MethodType methodType;
	private final int modifiers;
	private final int referenceKind;

	/*
	 * Creates a new instance of this class and initializes all its fields.
	 *
	 * @param declaringClass  The class that declares the method or field that this MethodHandleInfo will represent
	 * @param name  The name of the method or field that this MethodHandleInfo will represent
	 * @param methodType  A MethodType object representing the signature of the method or field that this MethodHandleInfo will represent
	 * @param modifiers  The modifiers of the method or field that this MethodHandleInfo will represent
	 * @param referenceKind  The reference kind of the method or field that this MethodHandleInfo will represent
	 */
	public MethodHandleInfoImpl(Class declaringClass, String name, MethodType methodType, int modifiers, int referenceKind) {
		this.declaringClass = declaringClass;
		this.name = name;
		this.methodType = methodType;
		this.modifiers = modifiers;
		this.referenceKind = referenceKind;
	}

	public Class getDeclaringClass() {
		return declaringClass;
	}

	public String getName() {
		return name;
	}

	public MethodType getMethodType() {
		return methodType;
	}

	public int getModifiers() {
		return modifiers;
	}

	public int getReferenceKind() {
		return referenceKind;
	}

	public String toString() {
		return super.toString() + "[" + getName() + getMethodType() +"]";
	}
}

